function estdensity = PsplineDen01(y,param);
%
%   Regression spline density estimate
%
%   USAGE:  estden = srs(y,param)
%
%		OUTPUT retruned as a structure called, e.g., estdensity
%	yhat	= estimated density at values of x
%	x	= estimation points 
%			(points where the density has been estimated)
%	gcv	= gcv values at values of penwt
%	imin	= index of penwt that minimizes gcv
%	df	= degree of freedom as value of penwt
%
%		INPUT (required)
%	y	= sample
%
%		INPUT (optional) as a structure called, e.g., param
%	min_y	= smallest value at which the density is estimated
%	max_y	= largest value at which the density is estimated
%	degree 	= degree of the spline
%	nknots	= number of knots
%	penwt	= penalty weight
%
%	Last edit: 2/4/02


if nargin < 2 ;
	param = '' ;
end ;



if isfield(param,'min_y') == 0 ;
	min_y = min(y) - range(y)/3 ;
else ;
	min_y = param.min_y ;
end ;

if isfield(param,'max_y') == 0 ;
	max_y = max(y) + range(y)/3 ;
else ;
	max_y = param.max_y ;
end ;

if isfield(param,'degree') == 0 ;
	degree = 2 ;
else ;
	degree = param.degree ;
end ;

if isfield(param,'nknots') == 0 ;
	nknots = 15 ;
else ;
	nknots = param.nknots ;
end ;

if isfield(param,'penwt') == 0 ;
	penwt = logspace(-10,5,35)' ;
else ;
	penwt = param.penwt ;
end ;

nbin = 200 ;
if length(y) == 1 ; y = y' ; end ;

n = length(y);
x=linspace(min_y,max_y,nbin)';
y = hist(y,x)' ;
supp_y = max_y - min_y;
coeff = nbin/(n*supp_y) ;
y = y .* coeff ;
knots=linspace(min_y,max_y,nknots+2)' ;
knots = knots(2:nknots+1) ;
xm = powerbasis(x,degree,knots) ;

id = diag([zeros(1,degree+1)  ones(1,nknots)]) ;
m = length(penwt) ;
dim_beta = size(xm,2) ;
beta = zeros(dim_beta,m) ;
yhat = zeros(nbin,m) ;
asr = zeros(m,1) ;
gcv = asr ;
trsd = asr ;

	for i=1:m ;
		if i==1 ;
		start = -log(max_y-min_y) ;
		beta0=[start;zeros(dim_beta-1,1)] ;
		niter = 200 ;
		else ;
		beta0 = beta(:,i-1) ;
		niter = 6 ;
		end ;

		for iter=1:niter ;
		mu = exp(xm*beta0) ;
		wxm = ((1./mu)*ones(1,size(xm,2))).*xm ;
		b = inv(xm'*wxm + penwt(i)*id) ;
		beta0 = b*(wxm'*(y-mu) + xm'*wxm*beta0) ;
		end ;
	xxb = wxm'*xm*b ;
	trsd(i) = trace(xxb) ;
	
	
	beta(:,i) = beta0 ;
	yhat(:,i) = exp(xm*beta(:,i)) ;
	asr(i) =  norm( (y-yhat(:,i)) )^2 ;

	end ;
gcv = asr./ (1-trsd./nbin).^2 ;

imin = min(find(  (gcv==min(gcv)) ) ) ;
df= trsd ;
yhat = yhat(:,imin) ;
beta = beta(:,imin) ;

estdensity = struct('yhat',yhat,'x',x,'beta',beta,'gcv',gcv, ...
	'imin',imin,'df',df) ;
